@@ -189,6 +189,8 @@ DEFINE_TEST(test_read_filter_program)
 DEFINE_TEST(test_read_filter_program_signature)
 DEFINE_TEST(test_read_filter_uudecode)
 DEFINE_TEST(test_read_filter_uudecode_base64)
+DEFINE_TEST(test_read_filter_uudecode_raw)
+DEFINE_TEST(test_read_filter_uudecode_base64_raw)
 DEFINE_TEST(test_read_format_7zip)
 DEFINE_TEST(test_read_format_7zip_bzip2)
 DEFINE_TEST(test_read_format_7zip_from_fd)
@@ -206,6 +208,7 @@ DEFINE_TEST(test_read_format_7zip_lzma2_arm)
 DEFINE_TEST(test_read_format_7zip_ppmd)
 DEFINE_TEST(test_read_format_7zip_lzma2_arm64)
 DEFINE_TEST(test_read_format_7zip_deflate_arm64)
+DEFINE_TEST(test_read_format_7zip_win_attrib)
 DEFINE_TEST(test_read_format_7zip_encryption_data)
 DEFINE_TEST(test_read_format_7zip_encryption_header)
 DEFINE_TEST(test_read_format_7zip_encryption_partially)
